import os
from osgeo import gdal
from osgeo import osr

import numpy as np

import exceptions

import multiprocessing

import pickle

#from scipy.ndimage import correlate

#from scipy.ndimage import fourier_uniform
#from scipy.ndimage import median_filter
#from scipy.ndimage import gaussian_filter
#
#from scipy.signal import correlate2d
#from scipy.ndimage import correlate

from skimage.morphology import binary_erosion

from skimage.transform import hough_circle





gdal.UseExceptions()

def read_image(img):
    dtm = gdal.Open(img)
    
    dtm_array = dtm.ReadAsArray()
    
    spatial = dtm.GetGeoTransform()
    
    proj = dtm.GetProjection()
    
    return {'proj':proj,
                  'spat':spatial,
                  'dtm':dtm_array,
                  'name':os.path.split(img)[-1].split('.')[0]}
  

def pixelPoint(px,
                       py,
                       spatial):
                    
    # minimum x, e-w pixel size, rotation, maximum y, n-s pixel size, rotation
    
    x = (px*spatial[1])+spatial[0]
    y = (py*spatial[3])+spatial[5]
    
    return x,y
    
def radii(min_rad,max_rad,spatial):
    radii = []
    
    r = (max_rad-min_rad)/spatial[1]
    
    b = min_rad/spatial[1]
    
    for i in range(int(r)):
        radii.append(b+(i*spatial[1]))

    return radii
    

def writeimage(outpath, 
               outname,
               spatial,
               image):
 
    data_out = image
    print('ROWS,COLS',image.shape)
    print('Call to write image')
    os.chdir(outpath)
    print('OUTPATH',outpath)
    print('OUTNAME',outname)
     
    #load the driver for the format of choice
    driver = gdal.GetDriverByName("Gtiff") 
    #create an empty output file
    #get the number of bands we'll need:
     
    if len(image.shape) >2:
        bands = image.shape[2]
    else:
        bands = 1
    
    print('BANDS OUT', bands)
     
    #file name, x columns, y columns, bands, dtype
    out = driver.Create(outname, 
                                 image.shape[1], 
                                 image.shape[0], 
                                 bands, 
                                 gdal.GDT_Int16,
                                 options=['COMPRESS=LZW'])
                                 
    #define the location using coords of top-left corner
    # minimum x, e-w pixel size, rotation, maximum y, n-s pixel size, rotation
    out.SetGeoTransform(spatial)
 
    srs = osr.SpatialReference()
    #get the coodrinate system using the ESPG code
    srs.SetWellKnownGeogCS("EPSG:25832")
    #set pstackedstackedstackedtojection of output file 
    out.SetProjection(srs.ExportToWkt())
     
    band = 1
     
    if bands == 1:
        out.GetRasterBand(band).WriteArray(data_out)
        #set the no data value
        out.GetRasterBand(band).SetNoDataValue(-999)
        #apend the statistics to dataset
        out.GetRasterBand(band).GetStatistics(0,1)
         
        print('Saving %s/%s' % (band,bands))
         
    else:
        while (band<=bands):
            data = data_out[:,:,band-1]
            #write values to empty array
            out.GetRasterBand(band).WriteArray( data )    
            #set the no data value
            out.GetRasterBand(band).SetNoDataValue(-999)
            #apend the statistics to dataset
            out.GetRasterBand(band).GetStatistics(0,1)  
            print('Saving %s/%s' % (band,bands))
            band = band+1 
    out = None    
     
    print('Processing of %s complete' % (outname))       
     
    return outname
    


def hough(image,
                  spatial,
                  template,
                  outdir,
                  outname):
             
             
    edges = image - binary_erosion(image)
    
    rad = radii(50,125,spatial)
    
    circs = hough_circle(edges,rad, full_output=True)
    
    os.chdir(outdir)
    
    pickle.dump(circs,  open(outname+'.p','wb'))
    
    pickle.dump(spatial, open(outname+'_spat.p'))

    peaks = hough_circle_peaks(circs,rad)
    
    pickle.dump(peaks, open(outname+'_peaks.p'))
    
    try:
        peak_array = np.column_stack(peaks)
        
        # accum, cx, cy, rad
       
        peak_array[:,1] = (peak_array [:,1]*spatial[1])+spatial[0]
      
        peak_array[:,2] = (peak_array [:,2]*spatial[3])+spatial[5]
        
        peak_array[:,3]=(peak_array[:,3]*spatial[1])
        
        np.savetxt(outname+'_points.csv', peak_array, delimtier=';')
      

    except:
        pass
        
    return 'File %s Done' %(outname)



if __name__ == '__main__':
    print ('test')

#in_d = r'E:\TEMP\bg_fft_test\in\591_641_6149_6199.tif'
#outdir  = r'E:\TEMP\bg_fft_test\out'


    outdir = r'C:\Users\au526889\Downloads\ringborg_finding\rr\fft_test\corr_test3'

    if not os.path.exists(outdir):
        os.mkdir(outdir)

    #in_d = r'C:\Users\au526889\Downloads\ringborg_finding\rr\fft_test\input'
    in_d = r'C:\Users\au526889\Downloads\ringborg_finding\rr\fft_test\self_test'

    temp_list = templates(templ)

    temp_images = temp_list[0]

    temp_names = temp_list[1]

    temp_l = len(temp_list[0])

    print (temp_l, len(temp_images), len(temp_names))

    n_threads = multiprocessing.cpu_count()-1

    print (in_d)


    for img in os.listdir(in_d):
        print (img)
        iname = img.split('.')[0]
        if img.split('.')[-1]=='tif':
            im = read_image(os.path.join(in_d,img))

            image = im['dtm']

            spatial = im['spat']


            pool = multiprocessing.Pool(processes=n_threads)

            results = [pool.apply_async(corr, args=(image,spatial,temp_images[x],outdir,'%s_%s.tif'%(iname,temp_names[x]))) for x in range(temp_l)]

            output = [p.get() for p in results]

            print (output)


    