# -*- coding: utf-8 -*-
"""
Created on Mon Apr 16 13:45:22 2018

@author: au526889
"""

import os
from osgeo import gdal
from osgeo import osr

import numpy as np

#from scipy.ndimage import correlate

from scipy.ndimage import fourier_uniform
from scipy.ndimage import median_filter
from scipy.ndimage import gaussian_filter

gdal.UseExceptions()

def read_image(img):
    dtm = gdal.Open(img)
    
    dtm_array = dtm.ReadAsArray()
    
    spatial = dtm.GetGeoTransform()
    
    proj = dtm.GetProjection()
    
    return {'proj':proj,
            'spat':spatial,
            'dtm':dtm_array,
            'name':os.path.split(img)[-1].split('.')[0]}

'''adfGeoTransform[0]/* top left x */
adfGeoTransform[1]/* w-e pixel resolution */
adfGeoTransform[2]/* rotation, 0 if image is "north up" */
adfGeoTransform[3]/* top left y */
adfGeoTransform[4]/* rotation, 0 if image is "north up" */
adfGeoTransform[5]/* n-s pixel resolution */'''

def normalise(inarray):
    n = (inarray-np.min(inarray))/(np.max(inarray)-np.min(inarray))
    
    return n

def invert(inarray):
    i = np.max(inarray)-inarray
    
    
    return i

def dist_thresh(inarray,
                dt = 10000,
                psize=7):
    
    
    trans = dt/psize
    
    m = inarray>trans
    
    o = inarray
    
    o[m]=trans
    
    return o

def proximity_raster(inarray):
    
    d = dist_thresh(inarray, psize=7)
    
    i = invert(d)
    
    n = normalise(i)
    
    return n

def terrain_raster(inarray,
                   no_data=-999):
    
    m = inarray==no_data
    
    o = inarray
    
    o[m]=0
    
    n = normalise(o)
    
    return n
    
    
def pt_pick(inarray,
            spatial,
            raster,
            xcol=0,
            ycol=1):
    
    print (spatial)

    x = inarray[:,xcol]    
    y = inarray[:,ycol]
    
    print (x,y)
    
    nr = np.shape(raster)[0]
    
    
    nc = np.shape(raster)[1]
    
    print (nr,nc)
        
    c = (x-spatial[0])/spatial[1]
    
    r = (y-spatial[3])/spatial[5]
    
    c=c.astype(int)
    r=r.astype(int)
    
    
    c[c>nc]=0
    
    r[r>nr]=0
    
    o = raster[r,c]

    return o

if __name__ == '__main__':
    
    tcols = []
    dcols = []
    
    tinpts = np.loadtxt(r'C:\Users\au526889\Dropbox\borgring_eval\nn_training\td.csv',delimiter=' ',skiprows=1)
    tinpts=tinpts[:,:2]
    
    dinpts = np.loadtxt(r'C:\Users\au526889\Dropbox\borgring_eval\nn_training\td.csv',delimiter=' ',skiprows=1)
    dinpts=dinpts[:,:2]
    
    
    
    indir = r'C:\Users\au526889\Dropbox\borgring_eval\landscape_context'
    
    odir = r'C:\Users\au526889\Dropbox\borgring_eval\nn\training_data'
    
    prox_list = [{'f':'alm_vej.tif','ps':7},
                 {'f':'bank.tif','ps':7},
                 {'f':'borg.tif','ps':7},
                 {'f':'borre.tif','ps':7},
                 {'f':'enge_soe_kyst.tif','ps':7},
                 {'f':'lv.tif','ps':7},
                 {'f':'ring','ps':7},
                 {'f':'trael.tif','ps':7},
                 {'f':'trelle.tif','ps':7},
                 {'f':'trold.tif','ps':7},
                 {'f':'vandloeb_prox.tif','ps':7}]
    
    terr_list = [{'f':'slope.tif', 'nd':-9999},
                 {'f':'tpi_dtm_7m_10_20.tif','nd':-999},
                 {'f':'tpi_dtm_7m_10_40.tif','nd':-999}]
    
    
    for i in prox_list:
        print (i)
        im = read_image(os.path.join(indir,i['f']))
        p = proximity_raster(im['dtm'])
        v = pt_pick(tinpts,im['spat'],p)
        n = os.path.join(odir,'t_'+i['f'].split('.')[0])
        np.save(n,v)
        tcols.append(n)
        
        v = pt_pick(dinpts,im['spat'],p)
        n = os.path.join(odir,i['f'].split('.')[0])
        np.save(n,v)
        dcols.append(n)
        
        
    for i in terr_list:
        im = read_image(os.path.join(indir,i['f']))
        p = terrain_raster(im['dtm'],no_data=i['nd'])
        v = pt_pick(tinpts,im['spat'],im['dtm'])
        n = os.path.join(odir,'t_'+i['f'].split('.')[0])
        np.save(n,v)
        tcols.append(n)     
        
        v = pt_pick(dinpts,im['spat'],im['dtm'])
        n = os.path.join(odir,i['f'].split('.')[0])
        np.save(n,v)
        dcols.append(n) 
        
    j = 0   
    
    td = None
        
    for i in tcols:
        if td is None:
            td = np.load(i)
        else:
            td=np.column_stack((td,np.load(i)))
    np.save(os.path.join(odir,'training_data'),td)
    
    dd = None
        
    for i in dcols:
        if dd is None:
            dd = np.load(i)
        else:
            dd=np.column_stack((dd,np.load(i)))
    np.save(os.path.join(odir,'data_data'),dd)
    
            
        
    
    
    