import os
from osgeo import gdal
from osgeo import osr

import numpy as np

#from scipy.ndimage import correlate

from scipy.ndimage import fourier_uniform
from scipy.ndimage import median_filter
from scipy.ndimage import gaussian_filter

gdal.UseExceptions()

def read_image(img):
    dtm = gdal.Open(img)
    
    dtm_array = dtm.ReadAsArray()
    
    spatial = dtm.GetGeoTransform()
    
    proj = dtm.GetProjection()
    
    return {'proj':proj,
                  'spat':spatial,
                  'dtm':dtm_array,
                  'name':os.path.split(img)[-1].split('.')[0]}
    
def writeimage(outpath, 
               outname,
               spatial,
               image):
 
    data_out = image
    print('ROWS,COLS',image.shape)
    print('Call to write image')
    os.chdir(outpath)
    print('OUTPATH',outpath)
    print('OUTNAME',outname)
     
    #load the driver for the format of choice
    driver = gdal.GetDriverByName("Gtiff") 
    #create an empty output file
    #get the number of bands we'll need:
     
    bands = 1
    print('BANDS OUT', bands)
     
    #file name, x columns, y columns, bands, dtype
    out = driver.Create(outname, image.shape[1], image.shape[0], bands, gdal.GDT_Float32)
    #define the location using coords of top-left corner
    # minimum x, e-w pixel size, rotation, maximum y, n-s pixel size, rotation
    out.SetGeoTransform(spatial)
 
    srs = osr.SpatialReference()
    #get the coodrinate system using the ESPG code
    srs.SetWellKnownGeogCS("EPSG:25832")
    #set pstackedstackedstackedtojection of output file 
    out.SetProjection(srs.ExportToWkt())
     
    band = 1
     
    if bands == 1:
        out.GetRasterBand(band).WriteArray(data_out)
        #set the no data value
        out.GetRasterBand(band).SetNoDataValue(-999)
        #apend the statistics to dataset
        out.GetRasterBand(band).GetStatistics(0,1)
         
        print('Saving %s/%s' % (band,bands))
         
    else:
        while (band<=bands):
            data = data_out[:,:,band-1]
            #write values to empty array
            out.GetRasterBand(band).WriteArray( data )    
            #set the no data value
            out.GetRasterBand(band).SetNoDataValue(-999)
            #apend the statistics to dataset
            out.GetRasterBand(band).GetStatistics(0,1)  
            print('Saving %s/%s' % (band,bands))
            band = band+1 
    out = None    
     
    print('Processing of %s complete' % (outname))       
     
    return outname
    


def rr_g(image_dict,
                out_dir,
                trend_diam = 10,
                pre_diam = 2,
                save_trend = True,
                save_pre = True,
                save_rr = True):
                    
    print (image_dict['dtm'].shape)
    
    name = image_dict['name']
                
    image = image_dict['dtm']

    trend = gaussian_filter(image,
                            sigma=trend_diam)
                                        
    if save_trend is True:
        print ('Saving trend image')
        writeimage(outdir,
                            't_%s.tif'%(name),
                            image_dict['spat'],
                            trend)
                                        
    pre = gaussian_filter(image, 
                          sigma=pre_diam)
    
    if save_pre is True:
        print ('saving prefiltered image')
        writeimage(outdir,
                            'p_%s.tif'%(name),
                            image_dict['spat'],
                            pre)
    
    rr = pre-trend
    
    if save_rr is True:
        print ('saving residual relief')
        
        writeimage(outdir,
                            'r_%s.tif'%(name),
                            image_dict['spat'],
                            rr)
        
    
outdir = '/run/media/dav/d_stott/testing/2007_new/30_KM_BLOCKS/rr'
in_d = r'/home/dav/data/2007_danmark'

for img in os.listdir(in_d):
    if img.split('.')[-1]=='tif':
        rr_g(read_image(img),outdir, save_trend=False,save_pre=False)
    
    