import os
from osgeo import gdal
from osgeo import osr

import numpy as np

import multiprocessing

from skimage.feature import match_template

from skimage.morphology import remove_small_objects
 
import pickle



gdal.UseExceptions()

def read_image(img):
    dtm = gdal.Open(img)
    
    dtm_array = dtm.ReadAsArray()
    
    spatial = dtm.GetGeoTransform()
    
    proj = dtm.GetProjection()
    
    return {'proj':proj,
                  'spat':spatial,
                  'dtm':dtm_array,
                  'name':os.path.split(img)[-1].split('.')[0]}
  
def writeimage(outpath, 
               outname,
               spatial,
               image):
 
    data_out = image
    print('ROWS,COLS',image.shape)
    print('Call to write image')
    os.chdir(outpath)
    print('OUTPATH',outpath)
    print('OUTNAME',outname)
     
    #load the driver for the format of choice
    driver = gdal.GetDriverByName("Gtiff") 
    #create an empty output file
    #get the number of bands we'll need:
     
    if len(image.shape) >2:
        bands = image.shape[2]
    else:
        bands = 1
    
    print('BANDS OUT', bands)
     
    #file name, x columns, y columns, bands, dtype
    out = driver.Create(outname, 
                                 image.shape[1], 
                                 image.shape[0], 
                                 bands, 
                                 gdal.GDT_Int16,
                                 options=['COMPRESS=LZW'])
                                 
    #define the location using coords of top-left corner
    # minimum x, e-w pixel size, rotation, maximum y, n-s pixel size, rotation
    out.SetGeoTransform(spatial)
 
    srs = osr.SpatialReference()
    #get the coodrinate system using the ESPG code
    srs.SetWellKnownGeogCS("EPSG:25832")
    #set pstackedstackedstackedtojection of output file 
    out.SetProjection(srs.ExportToWkt())
     
    band = 1
     
    if bands == 1:
        out.GetRasterBand(band).WriteArray(data_out)
        #set the no data value
        out.GetRasterBand(band).SetNoDataValue(-999)
        #apend the statistics to dataset
        out.GetRasterBand(band).GetStatistics(0,1)
         
        print('Saving %s/%s' % (band,bands))
         
    else:
        while (band<=bands):
            data = data_out[:,:,band-1]
            #write values to empty array
            out.GetRasterBand(band).WriteArray( data )    
            #set the no data value
            out.GetRasterBand(band).SetNoDataValue(-999)
            #apend the statistics to dataset
            out.GetRasterBand(band).GetStatistics(0,1)  
            print('Saving %s/%s' % (band,bands))
            band = band+1 
    out = None    
     
    print('Processing of %s complete' % (outname))       
     
    return outname

def temp_match(image,
               spatial,
               template,
               outdir,
               outname,
               done,
               match_thresh=0.35,
               save_bins=False,
               save_pts=True):
   
   print ('Call to matching')
    
   if not outname in done:
   
       match = match_template(image,template,pad_input=True)
       
       print (match)
       
       if save_bins is True:
           pickle.dump({'matches':match,'spat':spatial}, open(os.path.join(outdir,outname+'.p'),'wb'),protocol=4)
       
       if save_pts is True:
           
           pts = None
       
           match_idx = np.argwhere(match>0.35)
           
           if len(match_idx)>0:
    
               for i in match_idx:
                   x = (i[1]*spatial[1])+spatial[0]
                  
                   y = spatial[3]-(i[0]*np.abs(spatial[5]))
                   
                   val = match[i[0],i[1]]
                   
                   if pts is None:
                       pts = np.array((i[1],i[0],x,y,val))
                       
                   else:
                       pts=np.vstack((pts,np.array((i[1],i[0],x,y,val))))
                       
               np.savetxt(os.path.join(outdir,outname+'_points.csv'),pts, delimiter=';')
               
   return outname

def templates(template_dir):

    temp_i = []
    temp_n = []

    for temp in os.listdir(template_dir):
        if temp.split('.')[-1] == 'tif':

            tempn = temp.split('.')[0]

            temp_n.append(tempn)
            template = read_image(os.path.join(template_dir, temp))['dtm']

            temp_i.append(template)

    return temp_i, temp_n

if __name__ == '__main__':
    print ('test')

#in_d = r'E:\TEMP\bg_fft_test\in\591_641_6149_6199.tif'
#outdir  = r'E:\TEMP\bg_fft_test\out'


    outdir = r'/home/dav/data/Dropbox/borgring python/template_matching'

    if not os.path.exists(outdir):
        os.mkdir(outdir)

    #templ = r'O:\ST_Geoarkaeologi\Borgring SfM\David_Working\feature_detection\training_data\rings\self'
    #templ = r'/run/media/dav/d_stott/testing/rings/raster'
    templ = r'/home/dav/data/Dropbox/borgring python/new_rings'

    #in_d = r'C:\Users\au526889\Downloads\ringborg_finding\rr\fft_test\input'
    in_d = r'/run/media/dav/d_stott/testing/2007_new/30_KM_BLOCKS/thresh'

    temp_list = templates(templ)

    temp_images = temp_list[0]

    temp_names = temp_list[1]

    temp_l = len(temp_list[0])

    print (temp_l, len(temp_images), len(temp_names))

    #n_threads = multiprocessing.cpu_count()-1
    n_threads = 1

    print (in_d)


    for img in os.listdir(in_d):
        iname = img.split('.')[0]
        if img.split('.')[-1]=='tif':
            
            im = read_image(os.path.join(in_d,img))

            image = im['dtm']
            
            image = image>0
            image = remove_small_objects(image,min_size=128, connectivity=2)

            spatial = im['spat']
            
            done = []
            
            
            '''pool = multiprocessing.Pool(processes=n_threads)

            results = [pool.apply_async(temp_match, args=(image,spatial,temp_images[x],outdir,'%s_%s.tif'%(iname,temp_names[x]),done)) for x in range(temp_l)]'''
            
            #output = [p.get() for p in results]
            
            for i in range(temp_l):
               with open(os.path.join(outdir,'done.csv'), 'r') as logf:
                   done = logf.readlines()
                   logf.close()
                   
                   oname='%s_%s.tif'%(iname,temp_names[i])
                   
               if not oname in done:
            
                   print (img, temp_names[i])
                   t = temp_match(image, spatial, temp_images[i],outdir,'%s_%s.tif'%(iname,temp_names[i]),done)
                   

            
            
                   with open(os.path.join(outdir,'done.csv'), 'a+') as logf:
                        logf.write("%s\n" % t)
                        logf.close()
        
                    



    